/**********************************************************************************************/
/* FBL_BitSetFile.cpp																		  */
/*                                                                       					  */
/* Foundation Base Library ( FBL )		                                   					  */
/*                                                                       					  */
/* Copyright Paradigma, 1993-2002                                       					  */
/* All Rights Reserved.                                                 					  */
/**********************************************************************************************/

#ifndef __FBL_BitSetFile_h
   #define __FBL_BitSetFile_h

#include "FBL_EmbededFile.h"

#if defined(__MWERKS__)
	#pragma options align=power
#endif


/**********************************************************************************************/
#define WORD_SIZE 		sizeof(int)
#define BITS_IN_CHAR	8	


/**********************************************************************************************/
class FBL_BitSet;


/**********************************************************************************************/
class FBL_BitSetFile : public FBL_EmbededFile 
{
		friend class FBL_Table;
		friend class BF_BOOL;

	public://///////////////////////////////////////////////////////////////////////////////////

							FBL_BitSetFile( 
									FBL_DataFile* 	inDataFile, 
									ullong 			inName,
									ushort			inHeaderSize );
						
		/* ITEM OPERATIONS:	 */

		void				Set    ( ID_TYPE inValue, bool inContents );	// range 1..
		bool				Include( ID_TYPE inValue );						// range 1..
					
		ID_TYPE				First( void );
		ID_TYPE				Last( void );
		
		ID_TYPE				Next( void );
		ID_TYPE				Prev( void );
		
		void				GoTo( ID_TYPE inValue );
		
		ID_TYPE				GetFree( void );
		
							// Make length of the file such that it can store exactly inItems.
							// The BitSetFile, of course, align it to the byte.
		void				SetItemsCount( ID_TYPE inItems );

		FBL_BitSet*			GetBitSet( void );

	protected://////////////////////////////////////////////////////////////////////////////////

virtual	void				ClearValues( void );

		void				LocByte ( ID_TYPE inByte );
		
							// Returns Mask from static mArray.
							// & 7 - means cutting of low 3 bits from value.
		uchar				Mask( ID_TYPE inValue ) { return sMasks[ (uchar)(inValue & 0x07) ];}

		void				FirstBit( void );
		void				LastBit( void );
		bool 				NextBit( void );
		bool 				PrevBit( void );

		ulong				AllocatedBytes( void ) const { 
								flength Length = GetLength();
								return (Length <= mOffset) ? 0 : (ulong)(Length - mOffset); 
							}		

	protected://////////////////////////////////////////////////////////////////////////////////

		ushort				mOffset;			// for BF_Boolean it is 0.
												// for DeleteMap it is size of table header.
												// This is offset of bit vector in file.	
													
		uchar 				mByte;				// byte itself 
		ulong				mByteNumber;		// its number, 0..N-1
		uchar				mCurrBit;			// range [0..7]
		
static 	uchar     			sMasks[8];

};


/**********************************************************************************************/
inline void FBL_BitSetFile::GoTo( ID_TYPE inValue )
{
	if( inValue > 0 ){
		inValue--;
			
		LocByte( inValue >> 3 );
		mCurrBit = (uchar) (inValue & 0x07);
	}else{
		mByte		= 0;
		mCurrBit	= 0;
	}
}

/**********************************************************************************************/
inline bool FBL_BitSetFile::Include( ID_TYPE inValue )
{
	if( inValue == 0 || inValue > AllocatedBytes() * 8 )
		return false;
	
	inValue--;
		
	LocByte( inValue >> 3 );
	return mByte & Mask( inValue ) ? true : false;
	
} /* FBL_BitSetFile::Remove */


/**********************************************************************************************/
#if defined(__MWERKS__)
	#pragma options align=reset
#endif

#endif
