/**********************************************************************************************/
/* VDK_Link.h																		 	  	  */
/*                                                                       					  */
/* Valentina SDK    																		  */
/*                                                                       					  */
/* Copyright Paradigma, 1993-2002                                        					  */
/* All Rights Reserved.                                                 					  */
/**********************************************************************************************/

#ifndef __VDK_Link_h
   #define __VDK_Link_h

#if VSDK_FRAMEWORK_STYLE
#	include <VSDK/FBL.h>
#else
#	include <FBL.h>
#endif // VSDK_FRAMEWORK_STYLE

#if defined(__MWERKS__)
	#pragma options align=power
#endif

/**********************************************************************************************/
class FBL_BitSet;
class FBL_ArraySet;

class FBL_FileStream;
class FBL_String;
class OBL_Link;

class VDK_BaseObject;
class VDK_DataBase;


/**********************************************************************************************/
class VDK_Link 
{
    public://///////////////////////////////////////////////////////////////////////////////////

		/* Construction/destruction */
		
FBL_EXP							VDK_Link( void );				 
FBL_EXP virtual					~VDK_Link( void );

virtual	VDK_Link*				Clone( void )	{ return NULL;		}

		/* LINK ACCESSORIES: */	
	
virtual	FBL_Link_Type			GetLink_Type( void ) const									= 0;	
virtual	const char*				GetLinkName( void )	const									= 0;	
	

		/* BRANCHES METHODES: */
	
virtual	bool					AllowDeletionFor( VDK_BaseObject* inObject ) 				= 0;
								// Return true if deletion of record is allowed for inObject. 

virtual void					ChangeBranchParameters( 
										ushort 					inBranchNumber,
										FBL_RelType 			inNewRelType,
										FBL_DeletionControl 	inDeletionControl)  		= 0;
								// Cnhange parameters of existing branch.

virtual ushort					GetBranchesCount( void ) const								= 0;
								// Returns number of branches in relation.	
	
virtual void					GetBranchParameters( 
										ushort 					inBranchNumber, 
										FBL_RelType& 			outRelType,
										FBL_DeletionControl& 	outDeletionControl ) const 	= 0;
								// Returns relType for inBaseObject.
								// Returns FBL_Many if link has not this object.

		void					GetBranchParameters( 
										VDK_BaseObject*			inBaseObject, 
										FBL_RelType& 			outRelType,
										FBL_DeletionControl& 	outDeletionControl ) 
								{
									GetBranchParameters( GetObjectIndex(inBaseObject),
														 outRelType, outDeletionControl);
								}

virtual	const char*				GetNameFor( 
									VDK_BaseObject* inBaseObjectA,
									VDK_BaseObject* inBaseObjectB )	const					= 0;
								// Returns the name of specified link in direction from
								// inBaseObjectA to inBaseObjectB.

virtual VDK_BaseObject*			GetObject( ushort BranchNumber ) const						= 0;
								// Returns BaseObject for specified branch.

FBL_EXP	ushort					GetObjectIndex( VDK_BaseObject* inBaseObject, ushort inStartIndex = 1 );
		
		bool					Has( VDK_BaseObject* pBaseObject ) {
									return bool( GetObjectIndex(pBaseObject) > 0 );
								}

FBL_EXP	bool					IsBetween( 
										VDK_BaseObject* inBaseObjectA, 
										VDK_BaseObject* inBaseObjectB );
								// Returns true if this Link is between specified 2 objects.
								// This method works correctly for self-recurtion also.
		
		
		/* LINK RECORD Methods: */
										
virtual bool					AddLinkRecord( FBL_REC_ID Keys[] ) = 0;		
								// Add to relation new record with given set of keys.
								// Array must contains right count of keys,
								// in order of branches.

virtual void					RemoveLinkRecord( FBL_REC_ID Keys[] ) = 0;	
								// Remove from relation record wich contains 
								// given set of keys.
		
FBL_EXP	void 					BreakLinksOfRecord( 
										VDK_BaseObject*	inMainObject,
										FBL_REC_ID 		inMainRecID );

FBL_EXP	void 					BreakAllLinks( 
										VDK_BaseObject*	inMainObject );


		/* LINK SEARCH METHODES: */

FBL_EXP virtual ulong	 		GetLinkedRecordsCount( 
										VDK_BaseObject* inMainObject, 
										FBL_REC_ID		inMainRecID, 
										VDK_BaseObject* inRelatedObject );
								// Returns number of links { inMainRecID, inRec_B, ... }
								// IF (inRelatedObject == NULL) THEN returns number of all 
								// links { inMainRecID, ... }

FBL_EXP virtual FBL_ArraySet* 	GetList( 
										VDK_BaseObject* inMainObject, 
										FBL_REC_ID		inMainRecID, 
										VDK_BaseObject* inRelatedObject );
								// Find records in inRelatedObject linked with record
								// inMainRecID from inMainObject.

FBL_EXP virtual FBL_ArraySet* 	GetListOfMyOnly( 
										VDK_BaseObject* inMainObject, 
										FBL_REC_ID		inMainRecID, 
										VDK_BaseObject* inRelatedObject );
								// Find records in inRelatedObject linked with record
								// inMainRecID from inMainObject and only with it.


FBL_EXP virtual FBL_BitSet*		GetBitSetOfAllLinked(
										VDK_BaseObject* inMainObject,
										VDK_BaseObject* inRelatedObject );
								// Returns bit set of all records from inRelatedObject
								// which are linked to the records of inMainObject.


		/* Internal methods: */

		OBL_Link*				GetImplementation( void ) { return mLinkImp; }		

	protected:////////////////////////////////////////////////////////////////////////////////// 
	
		OBL_Link*				mLinkImp;

		VDK_DataBase* 			mDataBase;
};


/**********************************************************************************************/
#endif // __VDK_Link_h
